Solar_PanelPlaceable = {};
Solar_PanelPlaceable_mt = Class(Solar_PanelPlaceable, Placeable);

InitObjectClass(Solar_PanelPlaceable, "Solar_PanelPlaceable");

function Solar_PanelPlaceable:new(isServer, isClient, customMt)
    local mt = customMt;
    if mt == nil then
        mt = Solar_PanelPlaceable_mt;
    end;

    local self = Placeable:new(isServer, isClient, mt);

    registerObjectClassName(self, "Solar_PanelPlaceable");

    self.incomePerHour = 0;
    self.timer = math.random() * 2 * math.pi;

    self.speed = 0.00010;
    self.zRotationMin = 0;
    self.zRotationMax = Utils.degToRad(30);

    return self;
end;

function Solar_PanelPlaceable:delete()
    unregisterObjectClassName(self);
    g_currentMission.environment:removeHourChangeListener(self);
    Solar_PanelPlaceable:superClass().delete(self);
end;

function Solar_PanelPlaceable:deleteFinal()
    Solar_PanelPlaceable:superClass().deleteFinal(self);
end;

function Solar_PanelPlaceable:createNode(i3dFilename, moveMode)
    if not Solar_PanelPlaceable:superClass().createNode(self, i3dFilename, moveMode) then
        return false;
    end;

    self.head = getChildAt(self.nodeId, 1);
    self.cylinders = getChildAt(self.nodeId, 2);
    if self.head == 0 or self.cylinders == 0 then
        return false;
    end
    self.innerCylinders = getChildAt(self.cylinders, 0);
    if self.innerCylinders == 0 then
        return false;
    end

    return true;
end;

function Solar_PanelPlaceable:load(xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom)
    if not Solar_PanelPlaceable:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom) then
        return false;
    end;

    local xmlFile = loadXMLFile("TempXML", xmlFilename);

    local difficultyMultiplier = 2 ^ (3 - g_currentMission.missionStats.difficulty); --4 2 1
    self.incomePerHour = difficultyMultiplier * Utils.getNoNil(getXMLFloat(xmlFile, "placeable.incomePerHour"), 100);

    if not moveMode then
        g_currentMission.environment:addHourChangeListener(self);
    end

    delete(xmlFile);

    return true;
end;

function Solar_PanelPlaceable:update(dt)
    self.timer = self.timer + dt * 0.001;
    if self.timer >= 2 * math.pi then self.timer = 0; end;
    local sinValue = (math.sin(self.timer) + 1) / 2; -- renders a number that goes from 0 to 1 and back
    local zRotation = self.zRotationMax * sinValue;
    local fakeValue = 2.105 - 1.977 / math.cos(zRotation - self.zRotationMax / 2);
    local xRotation = Utils.degToRad(-fakeValue * 15);
    local yScale = 1 + 0.5 * sinValue;
    setRotation(self.head, 0, 0, zRotation);
    setRotation(self.cylinders, 0, 0, xRotation);
    setScale(self.innerCylinders, 1, yScale, 1);
end;

--[[function Solar_PanelPlaceable:updateTick(dt)
end;]]

function Solar_PanelPlaceable:hourChanged()
    if self.isServer then
        g_currentMission:addSharedMoney(self.incomePerHour, "other");
    end
end

registerPlaceableType("Solar_PanelPlaceable", Solar_PanelPlaceable);
